/* mbed GPS Module Library
 * Copyright (c) 2008-2010, sford
 * Copyright (c) 2013, B.Adryan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
 
#include "GPS.h"
 int i ;
GPS::GPS(PinName tx, PinName rx, int Baud,DigitalOut *gps_off) : _gps(tx, rx)
{
    _gps.baud(Baud);
    longitude = 0.0;
    latitude = 0.0;
    this->gps_off = gps_off;
    //_gps.printf("$$PMTK161,0*28\r\n");  //sleep
    _gps.printf("$$PMTK161,1*28\r\n");  //wake up
}
 
void GPS::idle(bool idle) 
 {     
    this->is_idle = idle; 
    gps_off->write(idle);
 }
  
  
void GPS::set_manual_tracking(bool mt) {     
 this->manual_tracking = mt; 
        printf("\n[GPS] Manual Tracking %x",mt); 
        gps_off->write(!mt);
  }
  
int GPS::sample()
{ 
    int lock;
    int h; 
    
    lock = 0;   
    for (h = 0; h < 9; h++)  {
        printf("\n[GPS] Line %i:",h); 
        getline();  
        
        if(sscanf(msg, "GPGGA,%f,%f,%c,%f,%c,%d,%d,%f,%f,%c,%f", &time, &latitude, &ns, &longitude, &ew, &lock, &sats, &hdop, &alt, &unit, &geoid)>=1) {
            printf(msg);
            if(!lock) {  
                printf("\n[GPS] Kein Lock: %f %ih %im %is\n",time,hour,minute,seconed); 
                time = 0.0;
                longitude = 0.0;
                latitude = 0.0;
                sats = 0;
                //hdop = 0.0;
                alt = 0.0;
                geoid = 0.0;
                GPS_signal_okay=0;
            } else {
                //GPGGA format according http://aprs.gids.nl/nmea/#gga
                // time (float), lat (f), (N/S) (c), long (f), (E/W) (c), fix (d), sats (d),
                // hdop (float), altitude (float), M, geoid (float), M, , ,
                //GPGGA,092010.000,5210.9546,N,00008.8913,E,1,07,1.3,9.7,M,47.0,M,,0000*5D
 
                //format utc time to beijing time,add 8 time zone
                
                hour = int(time) / 10000;
                minute = (int(time) % 10000) / 100;
                seconed = int(time) % 100;
                
                printf("\n[GPS] Lock: %ih %im %is lat:%, lon:%\n",hour,minute,seconed,latitude, longitude); 
                GPS_signal_okay=1;
                return lock ;      
            }//endiflock
        }    //endifscan        
    }   //endfor        
        return lock ;      
}
 
 
float GPS::trunc(float v)
{
    if(v < 0.0) {
        v*= -1.0;
        v = floor(v);
        v*=-1.0;
    } else {
        v = floor(v);
    }
    return v;
}
 
void GPS::getline()
{       
    while(_gps.getc() != '$');    // wait for the start of a line
    for(int i=0; i<256; i++) {
        msg[i] = _gps.getc();
        //printf("%c",msg[i]); 
        if(msg[i] == '\r') {
            msg[i] = 0;   
            return;
        }
    }
    error("GPS::getline Overflowed message limit");
}


void GPS::get_Position()
{  


    if (manual_tracking or !is_idle){
         gps_off->write(0);
         printf("\n[GPS] active");
         if (this->sample()) {  
            printf("\n[GPS] Position: %f , %f\n", longitude,latitude);
            }
     }
     else {
         gps_off->write(1);
         printf("\n[GPS] idle");    
    }
}


